/*! ========================================================================
** Extended Template Library
** Curve Class Implementation
** $Id$
**
** Copyright (c) 2002 Robert B. Quattlebaum Jr.
**
** This package is free software; you can redistribute it and/or
** modify it under the terms of the GNU General Public License as
** published by the Free Software Foundation; either version 2 of
** the License, or (at your option) any later version.
**
** This package is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
** General Public License for more details.
**
** === N O T E S ===========================================================
**
** This is an internal header file, included by other ETL headers.
** You should not attempt to use it directly.
**
** ========================================================================= */

/* === S T A R T =========================================================== */

#ifndef __ETL__CURVE_H
#define __ETL__CURVE_H

/* === H E A D E R S ======================================================= */

#include "_hermite.h"
#include <vector>

/* === M A C R O S ========================================================= */

/* === T Y P E D E F S ===================================================== */

/* === C L A S S E S & S T R U C T S ======================================= */

_ETL_BEGIN_NAMESPACE
/*
#ifdef AUTO
#undef AUTO
#endif
#ifdef LINEAR
#undef LINEAR
#endif
#ifdef ABSOLUTE
#undef ABSOLUTE
#endif
#ifdef FAST_TO_SLOW
#undef FAST_TO_SLOW
#endif

template<typename T>
class curve
{
public:
	typedef T value_type;
private:
	enum interpolate_type
	{
		AUTO,
		LINEAR,
		ABSOLUTE,
		FAST_TO_SLOW,

	};
public:
};
*/
_ETL_END_NAMESPACE

/* === E X T E R N S ======================================================= */

/* === E N D =============================================================== */

#endif

